#!/usr/bin/env python

##########
# import #
##########
import argparse
import logging
import os
import re
import sys

from py_tweet_format import __script_name__, __version__, count_word_chars, tokenize_string_to_list, words

def main():
	#############
	# arg parse #
	#############
	parser = argparse.ArgumentParser( prog = __script_name__, epilog = "%s v%s" % ( __script_name__, __version__ ) )

	parser.add_argument( 'input_file', help = 'Path to the input text file' )
	parser.add_argument( 'output_file', help = 'Path to the output file of Tweets' )
	parser.add_argument( '--shortened_url_length', help = 'Length of URLs after Twitter shortening', type = int, default = 23 )
	parser.add_argument( '--max_chunk_length', help = 'Maximum number of characters in a Tweet', type = int, default = 280 )
	parser.add_argument( '--keep_newlines', help = 'Keep newline characters in input document', default = False, action = 'store_true' )
	parser.add_argument( '--keep_extra_spaces', help = 'Keep extra spaces', default = False, action = 'store_true' )
	parser.add_argument( '--no_numbers', help = "Don't use numbering with each Tweet", default = False, action = 'store_true' )
	# NOT IMPLEMENTED parser.add_argument( '--use_x_of_y', help = 'Number Tweets using # of Total format', default = False, action = 'store_true' )
	parser.add_argument( "--loglevel", choices=[ 'DEBUG', 'INFO', 'WARNING', 'ERROR', 'CRITICAL' ], default='INFO' )

	args = parser.parse_args()

	#################
	# setup logging #
	#################
	logging.basicConfig( format = '%(asctime)s - %(name)s - %(levelname)s - %(message)s' )
	logger = logging.getLogger( __script_name__ )
	logger.setLevel( args.loglevel )

	###########################################
	# validity testing and additional parsing #
	###########################################
	## input != output
	input_file_path = os.path.normpath( args.input_file )
	output_file_path = os.path.normpath( args.output_file )

	if input_file_path == output_file_path:
		logger.error( "Program output would clobber input file" )
		sys.exit( 1 )

	## shortened url > 1
	if args.shortened_url_length < 1:
		logger.error( "Shortened URLs must be 1 or more characters" )
		sys.exit( 1 )

	## chunk size > 1
	if args.max_chunk_length < 1:
		logger.error( "Maximum allowable chunk is less than 1" )
		sys.exit( 1 )

	## compare chunks and shortened URLs
	if args.shortened_url_length > args.max_chunk_length:
		logger.error( "Shortened URLs exceed maximum Tweet length" )
		sys.exit( 1 )

	## not no numbers and use x of y
	# NOT IMPLEMENTED if args.no_numbers and args.use_x_of_y:
	#	logger.error( "Can't use no numbers and X of Y counting together" )
	#	sys.exit( 1 )

	###################
	# Options to info #
	###################
	options_set =  "\n%s v%s\n\n" % ( __script_name__, __version__ )
	options_set += "Options\n=======\n"
	options_set += "Input file: %s\n" % ( input_file_path )
	options_set += "Output file: %s\n" % ( output_file_path )
	options_set += "Size of each chunk: %s\n" % ( args.max_chunk_length )
	options_set += "Size of short URLs: %s\n" % ( args.shortened_url_length )
	options_set += "Remove newline characters: %s\n" % ( str( not args.keep_newlines ) )
	options_set += "Remove extra spaces: %s\n" % ( str( not args.keep_extra_spaces ) )
	options_set += "Use numbering: %s\n" % ( str( not args.no_numbers ) )
	# NOT IMPLEMENTED if args.no_numbers is False:
	#	options_set += "Use X of Y numbering: %s\n" % ( str( args.use_x_of_y ) )
	options_set += "Logging level: %s\n" % ( str( args.loglevel ) )

	logger.info( options_set )

	###################
	# slurp and clean #
	###################
	with open( input_file_path, 'r' ) as input_filehandle:
		text = input_filehandle.read()

	text = re.sub( "\t", " ", text )
	text = re.sub( "[ \n]+$", "", text )

	if not args.keep_newlines:
		text = re.sub( "\n", " ", text )
		text = re.sub( "\r", " ", text )

	if not args.keep_extra_spaces:
		text = re.sub( " +", " ", text )
		text = text.strip()

	#####################################
	# convert string to a list by token #
	#####################################
	parsed_strings = tokenize_string_to_list( text )

	################################################
	# Take the parsed strings and convert to words #
	################################################
	list_of_words = [ words( x, args.shortened_url_length ) for x in parsed_strings ]

	#####################################
	# parse the tweets into output file #
	#####################################
	with open( output_file_path, 'w' ) as OUT:
		current_tweet = 1
		first_bit = True

		if args.no_numbers:
			tweet = ""
		else:
			tweet = "%s) " %( current_tweet )

		idx = 0
		while idx < len( list_of_words ):
			current_word = list_of_words[ idx ]

			if len( tweet ) + current_word.size > args.max_chunk_length:
				tweet = re.sub( "[ \n]+$", "", tweet )
				OUT.write( "%s\n" % ( tweet ) )

				current_tweet += 1
				first_bit = True

				if args.no_numbers:
					tweet = ""
				else:
					tweet = "%s) " %( current_tweet )

			elif idx == len( list_of_words ) - 1:
				tweet += current_word.string
				tweet = re.sub( "[ \n]+$", "", tweet )
				OUT.write( "%s\n" % ( tweet ) )
				idx += 1
				
			else:
				if first_bit is True and current_word.string in [ " ", "\n" ]:
					idx += 1
					continue

				if first_bit is True:
					first_bit = False

				tweet += current_word.string

				idx += 1

if __name__ == "__main__":
	main()



#!/usr/bin/env python

"""
py_tweet_format: a tool for converting a text file into a set of tweets
"""

import re
import sys

####################
# Version and name #
####################
__script_path__ = sys.argv[0]
__script_name__ = __script_path__.split('/')[-1].split('\\')[-1]
__version__ = '1.0.0'

#############
# Functions #
#############
def count_word_chars( text, shortened_url_size ):
	"""
	A function to count the length of a string
	taking into account Twitters automatic
	URL shortening.

	Will detect a URL in strings that:
	1) Start with http, https, or ftp followed
	   by :// or
	2) Strings ending in one of the original
	   top-level domains
	"""
	if not isinstance( text, str ):
		raise TypeError( "count_word_chars function requires string input" )

	url = re.compile( u'^(https{0,1}|ftp)\:\/\/', re.IGNORECASE )
	pre_icann_tld = re.compile( u'\.(com|org|net|int|edu|gov|mil)$', re.IGNORECASE )

	if url.match( text ):
		return shortened_url_size
	elif pre_icann_tld.match( text ):
		return shortened_url_size
	else:
		return len( text )

def tokenize_string_to_list( full_string, tokens = [ ' ', '\n' ] ):
	index = 0
	start_at = 0
	end_at = 0

	string_size = len( full_string )

	string_list = []

	while start_at < string_size and end_at < string_size:
		if full_string[ end_at ] in tokens:
			string_list.append( full_string[ start_at:end_at ] )
			string_list.append( full_string[ end_at:end_at+1 ] )
			start_at = end_at + 1
		elif end_at == string_size - 1:
			string_list.append( full_string[ start_at:string_size ] )

		end_at += 1

	return string_list

###########
# Classes #
###########
class words:
	"""
	Class for storing words and the size of words.
	Important because URLs could be quite long, but
	may be truncated due to URL shortening. Making it
	a class enables list storage fo easier processing.
	"""
	def __init__( self, string, shortened_url_size ):
		if not isinstance( string, str ):
			raise TypeError( "words class requires the word to be a string" )

		if not isinstance( shortened_url_size, int ):
			raise TypeError( "shortened URL size must be an int" )

		self.string = string
		self.size = count_word_chars( self.string, shortened_url_size )

	def __str__( self ):
		print( "String: %s" % ( self.string ) )
		print( "Size: %s" % ( self.size ) )
		#return self.string

	def __repr__( self ):
		str( self )

########
# main #
########
if __name__ == "__main__":
	pass



from setuptools import setup

setup(
	name = "py_tweet_format",
	packages = ['py_tweet_format'],
	version = "1.0.0",
	description = 'Tool for formatting text files into tweets',
	author = "Elisha Roberson",
	author_email = 'dr.eli.roberson@gmail.com',
	url = 'https://github.com/eroberson/py_tweet_format',
	license = 'MIT',
	classifiers=[
	"Development Status :: 5 - Production/Stable",
	"Environment :: Console",
	"Intended Audience :: End Users/Desktop",
	"License :: OSI Approved :: MIT License",
	"Topic :: Internet",
	"Topic :: Text Processing",
	"Topic :: Utilities",
	"Programming Language :: Python :: 3",
	"Programming Language :: Python :: 3.2",
	"Programming Language :: Python :: 3.3",
	"Programming Language :: Python :: 3.4",
	"Programming Language :: Python :: 3.5",
	"Programming Language :: Python :: 3.6",
	"Programming Language :: Python :: 3.7",
	"Programming Language :: Python :: 3.8",
	"Programming Language :: Python :: 3.9",
	"Programming Language :: Python :: 3.10"
	],
	keywords='twitter tweet text format',
	entry_points = {'console_scripts':["py_tweet_format = py_tweet_format.__main__:main"]},
	test_suite = 'nose.collector',
	tests_require = ['nose']
)
